<?php
/**
*
* @package acp
* @copyright (c) 2005 phpBB Group, 2008 Justin Aquadro
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
* @package acp
*/
class acp_badges
{
	var $u_action;

	function main($id, $mode)
	{
		global $db, $user, $auth, $template, $cache;
		global $config, $phpbb_root_path, $phpbb_admin_path, $phpEx;
		
		$user->add_lang('mods/badges');
		
		$action = request_var('action', '');
		$action = (isset($_POST['add'])) ? 'add' : $action;
		$action = (isset($_POST['edit'])) ? 'edit' : $action;
		
		$mode = ($mode == 'badges') ? 'badges' : 'users';

		$this->tpl_name = 'acp_badges';

		$this->page_title = 'ACP_MANAGE_BADGES';

		// Action switch
		switch ($mode)
		{
			case 'badges':
				switch ($action)
				{
					case 'edit':
					case 'add':
						$this->show_edit_badge($action);
					break;
					
					case 'create':
					case 'modify':
						$this->create_modify($action);
					break;
					
					case 'delete':
						$notice = $this->delete_badge($id, $mode);
						$this->list_badges($action, $notice);
					break;
					
					case 'move_up':
					case 'move_down':
						$this->move_up_down($action);
						$this->list_badges($action);
					break;
					
					default:
						$this->list_badges($action);
					break;
				}
			break;
			
			case 'users':
				$this->show_user_badges();
			break;
		}
	}
	
	function list_badges($action, $notice = '')
	{
		global $db, $user, $auth, $template, $cache;
		global $config, $phpbb_root_path, $phpbb_admin_path, $phpEx;
		
		include_once($phpbb_root_path . 'includes/functions_badges.' . $phpEx);
		
		$_imageset = badge_imageset();
		
		// By default, check that order is valid and fix it if necessary
		$sql = 'SELECT badge_id AS order_id, badge_order AS fields_order
			FROM ' . BADGES_TABLE . '
			ORDER BY badge_order';
		$result = $db->sql_query($sql);
		
		if ($row = $db->sql_fetchrow($result))
		{
			$order = 0;
			do
			{
				++$order;
				if ($row['fields_order'] != $order)
				{
					$db->sql_query('UPDATE ' . BADGES_TABLE . "
						SET badge_order = $order
						WHERE badge_id = " . $row['order_id']);
				}
			}
			while ($row = $db->sql_fetchrow($result));
		}
		$db->sql_freeresult($result);
		
		$template->assign_vars(array(
			'L_TITLE'			=> $user->lang['ACP_BADGES'],
			'L_EXPLAIN'			=> $user->lang['ACP_BADGES_EXPLAIN'],
			'L_BADGE_ADD'		=> $user->lang['ADD_BADGES'],
			'L_BADGE_EDIT'		=> $user->lang['EDIT_BADGES'],
			
			'NOTICE'			=> $notice,
			'COLSPAN'			=> 3,
			
			'U_ACTION'			=> $this->u_action,
			)
		);

		$sql = $db->sql_build_query('SELECT', array(
			'SELECT'	=> 'b.*, g.group_name, g.group_type',
			'FROM'		=> array(
				BADGES_TABLE	=> 'b',
			),
			'LEFT_JOIN'	=> array(
				array(
					'FROM'	=> array(GROUPS_TABLE => 'g'),
					'ON'	=> 'b.badge_group = g.group_id'
				)
			),
			'ORDER_BY'	=> 'b.badge_order'
		));
		
		$result = $db->sql_query($sql);
		
		while ($row = $db->sql_fetchrow($result))
		{
			$def_only = empty($row['badge_group_default_only']) ? '' : ' (default-only)';
			
			if ($row['group_type'] == GROUP_SPECIAL)
			{
				$row['group_name'] = $user->lang['G_' . $row['group_name']];
			}
			
			if (empty($row['group_name']))
			{
				$row['group_name'] = '-----';
			}
			
			$row['group_name'] = str_replace(' ', '&nbsp;', $row['group_name']);
			
			$imglist = join(' ', get_badge_imageset($_imageset[$row['badge_url']], $row, 0));
			
			$template->assign_block_vars('items', array(
				'ALT_TEXT'		=> $row['badge_alt'],
				'TITLE_TEXT'	=> $row['badge_title'],
				'IMG_SRC'		=> $phpbb_root_path . $config['badge_path'] . '/' . $row['badge_url'],
				'IMG_SET'		=> $imglist,
				'WIDTH'			=> $row['badge_width'],
				'HEIGHT'		=> $row['badge_height'],
				'GROUP'			=> empty($row['badge_group']) ? '' : $row['group_name'] . $def_only,
				'U_EDIT'		=> $this->u_action . '&amp;action=edit&amp;id=' . $row['badge_id'],
				'U_DELETE'		=> $this->u_action . '&amp;action=delete&amp;id=' . $row['badge_id'],
				'U_MOVE_UP'		=> $this->u_action . '&amp;action=move_up&amp;id=' . $row['badge_id'],
				'U_MOVE_DOWN'	=> $this->u_action . '&amp;action=move_down&amp;id=' . $row['badge_id'],
				)
			);
		}
		$db->sql_freeresult($result);
	}
	
	function show_edit_badge($action)
	{
		global $db, $user, $auth, $template, $cache;
		global $config, $phpbb_root_path, $phpbb_admin_path, $phpEx;
		
		include_once($phpbb_root_path . 'includes/functions_badges.' . $phpEx);
		
		$icon_id = request_var('id', 0);

		$_imageset = array();
		
		// Get list of badges
		if ($action == 'add')
		{
			$_imageset = badge_imageset();
		}
		
		// Get a list of groups
		$sql = 'SELECT group_id, group_name, group_type
			FROM ' . GROUPS_TABLE . '
			ORDER BY group_name';
		$result = $db->sql_query($sql);
		
		$groups = array();
		$group_opts = array();
		
		$group_opts[0] = '<option value="0">-----</option>';
		
		while ($row = $db->sql_fetchrow($result))
		{
			if ($row['group_type'] == GROUP_SPECIAL)
			{
				$row['group_name'] = $user->lang['G_' . $row['group_name']];
			}
			$groups[$row['group_id']] = $row['group_name'];
			$group_opts[$row['group_id']] = '<option value="' . $row['group_id'] . '">' . $row['group_name'] . '</option>';
		}
		$db->sql_freeresult($result);

		// Get the badges as defined in the database
		$sql = 'SELECT *
			FROM ' . BADGES_TABLE . '
			ORDER BY badge_order ' . (($icon_id || $action == 'add') ? 'DESC' : 'ASC');
		$result = $db->sql_query($sql);
		
		$data = array();
		$after = false;
		$order_list = '';
		
		// Remove images of the _x# form
		foreach ($_imageset as $key => $img)
		{
			if (isset($img[1]))
			{
				$_imageset[$key] = $img[1];
			}
			else
			{
				unset($_imageset[$key]);
			}
		}
		
		while ($row = $db->sql_fetchrow($result))
		{
			if ($action == 'add')
			{
				unset($_imageset[$row['badge_url']]);
			}
			
			if ($row['badge_id'] == $icon_id)
			{
				$after = true;
				$data[$row['badge_url']] = $row;
			}
			else
			{
				if ($action == 'edit' && !$icon_id)
				{
					$data[$row['badge_url']] = $row;
				}
				
				$selected = '';
				if (!empty($after))
				{
					$selected = ' selected="selected"';
					$after = false;
				}
				
				$after_txt = $row['badge_url'];
				$order_list = '<option value="' . ($row['badge_order'] + 1) . '"' . $selected . '>' . sprintf($user->lang['AFTER_BADGES'], ' -&gt; ' . $after_txt) . '</option>' . $order_list;
			}
		}
		$db->sql_freeresult($result);
		
		$order_list = '<option value="1"' . ((!isset($after)) ? ' selected="selected"' : '') . '>' . $user->lang['FIRST'] . '</option>' . $order_list;

		if ($action == 'add')
		{
			$data = $_imageset;
		}
		
		$colspan = 8;
		$colspan += ($icon_id) ? 1 : 0;
		$colspan += ($action == 'add') ? 2 : 0;
		
		$template->assign_vars(array(
			'S_EDIT'		=> true,
			'S_ADD'			=> ($action == 'add') ? true : false,
			'S_ORDER_LIST'	=> $order_list,
			
			'L_TITLE'			=> $user->lang['ACP_BADGES'],
			'L_EXPLAIN'			=> $user->lang['ACP_BADGES_EXPLAIN'],
			'L_CONFIG'			=> $user->lang['BADGE_CONFIG'],
			'L_URL'				=> $user->lang['BADGE_URL'],
			'L_LOCATION'		=> $user->lang['BADGE_LOCATION'],
			'L_WIDTH'			=> $user->lang['BADGE_WIDTH'],
			'L_HEIGHT'			=> $user->lang['BADGE_HEIGHT'],
			'L_ALT_TEXT'		=> $user->lang['BADGE_ALT'],
			'L_TITLE_TEXT'		=> $user->lang['BADGE_TITLE'],
			'L_GROUP'			=> $user->lang['BADGE_GROUP'],
			'L_GROUP_DEFAULT'	=> $user->lang['BADGE_GROUP_DEFAULT'],
			'L_ORDER'			=> $user->lang['BADGE_ORDER'],
			'L_NO_ICONS'		=> $user->lang['NO_BADGE_' . strtoupper($action)],
			
			'COLSPAN'		=> $colspan,
			'ID'			=> $icon_id,
			
			'U_BACK'		=> $this->u_action,
			'U_ACTION'		=> $this->u_action . '&amp;action=' . (($action == 'add') ? 'create' : 'modify'),
		));
		
		foreach ($data as $img => $img_row)
		{
			$group_id = (isset($img_row['badge_group'])) ? $img_row['badge_group'] : 0;
			$local_group_list = '';
			
			if (!empty($group_opts[$group_id]) && $group_id > 0)
			{
				$group_opts[$group_id] = '<option value="' . $group_id . '" selected="selected">' . $groups[$group_id] . '</option>';
				$local_group_list = join("\n", $group_opts);
				$group_opts[$group_id] = '<option value="' . $group_id . '">' . $groups[$group_id] . '</option>';
			}
			else
			{
				$local_group_list = join("\n", $group_opts);
			}
				
			$template->assign_block_vars('items', array(
				'IMG'		=> $img,
				'A_IMG'		=> addslashes($img),
				'IMG_SRC'	=> $phpbb_root_path . $config['badge_path'] . '/' . $img,

				'S_ID'				=> (isset($img_row['badge_id'])) ? true : false,
				'ID'				=> (isset($img_row['badge_id'])) ? $img_row['badge_id'] : 0,
				'WIDTH'				=> (!empty($img_row['badge_width'])) ? $img_row['badge_width'] : $img_row['width'],
				'HEIGHT'			=> (!empty($img_row['badge_height'])) ? $img_row['badge_height'] : $img_row['height'],
				'TITLE'				=> (isset($img_row['badge_id'])) ? $img_row['badge_title'] : '',
				'ALT'				=> (isset($img_row['badge_id'])) ? $img_row['badge_alt'] : '',
				'GROUP'				=> $local_group_list,
				'GROUP_DEFAULT'		=> (!empty($img_row['badge_group_default_only'])) ? 'checked="checked"' : '',
			));
		}
	}
	
	function create_modify($action)
	{
		global $db, $user, $auth, $template, $cache;
		global $config, $phpbb_root_path, $phpbb_admin_path, $phpEx;
		
		// Get items to create/modify
		$images = (isset($_POST['image'])) ? array_keys(request_var('image', array('' => 0))) : array();
		
		// Now really get the items
		$image_id		= (isset($_POST['id'])) ? request_var('id', array('' => 0)) : array();
		$image_order	= (isset($_POST['order'])) ? request_var('order', array('' => 0)) : array();
		$image_width	= (isset($_POST['width'])) ? request_var('width', array('' => 0)) : array();
		$image_height	= (isset($_POST['height'])) ? request_var('height', array('' => 0)) : array();
		$image_add		= (isset($_POST['add_img'])) ? request_var('add_img', array('' => 0)) : array();
		$image_title	= utf8_normalize_nfc(request_var('title', array('' => ''), true));
		$image_alt		= utf8_normalize_nfc(request_var('alt', array('' => ''), true));
		$image_group	= (isset($_POST['group'])) ? request_var('group', array('' => 0)) : array();
		$image_group_default = (isset($_POST['group_default'])) ? request_var('group_default', array('' => 0)) : array();
		
		$icons_updated = 0;
		$errors = array();
		foreach ($images as $image)
		{
			if ($action == 'create' && !isset($image_add[$image]))
			{
				// skip images where add wasn't checked
				continue;
			}
			
			if ($image_width[$image] == 0 || $image_height[$image] == 0)
			{
				$img_size = getimagesize($phpbb_root_path . $config['badge_path'] . '/' . $image);
				$image_width[$image] = $img_size[0];
				$image_height[$image] = $img_size[1];
			}
			
			$img_sql = array(
				'badge_url'		=> $image,
				'badge_width'	=> $image_width[$image],
				'badge_height'	=> $image_height[$image],
				'badge_title'	=> $image_title[$image],
				'badge_alt'		=> $image_alt[$image],
				'badge_group'	=> (!empty($image_group[$image])) ? $image_group[$image] : 0,
				'badge_group_default_only'	=> (isset($image_group_default[$image])) ? 1 : 0,
			);
			
			// Image_order holds the 'new' order value
			if (!empty($image_order[$image]))
			{
				$img_sql = array_merge($img_sql, array(
					'badge_order'	=>	$image_order[$image])
				);

				// Since we always add 'after' an item, we just need to increase all following + the current by one
				$sql = 'UPDATE ' . BADGES_TABLE . "
					SET badge_order = badge_order + 1
					WHERE badge_order >= {$image_order[$image]}";
				$db->sql_query($sql);

				// If we adjust the order, we need to adjust all other orders too - they became inaccurate...
				foreach ($image_order as $_image => $_order)
				{
					if ($_image == $image)
					{
						continue;
					}

					if ($_order >= $image_order[$image])
					{
						$image_order[$_image]++;
					}
				}
			}
			
			if ($action == 'modify' && !empty($image_id[$image]))
			{
				$sql = 'UPDATE ' . BADGES_TABLE . '
					SET ' . $db->sql_build_array('UPDATE', $img_sql) . '
					WHERE badge_id = ' . $image_id[$image];
				$db->sql_query($sql);
				$icons_updated++;
			}
			else if ($action !== 'modify')
			{
				$sql = 'INSERT INTO ' . BADGES_TABLE . $db->sql_build_array('INSERT', $img_sql);
				$db->sql_query($sql);
				$icons_updated++;
			}
		}
		
		$cache->destroy('_badges');
		$cache->destroy('_badge_images');
		$cache->destroy('sql', BADGES_TABLE);
		
		$level = E_USER_NOTICE;
		switch ($icons_updated)
		{
			case 0:
				$suc_lang = 'BADGE_NONE';
				$level = E_USER_WARNING;
				break;
				
			case 1:
				$suc_lang = 'BADGE_ONE';
				break;
				
			default:
				$suc_lang = 'BADGE';
		}
		
		$errormsgs = '<br />';
		foreach ($errors as $img => $error)
		{
			$errormsgs .= '<br />' . sprintf($user->lang[$error], $img);
		}
		if ($action == 'modify')
		{
			trigger_error($user->lang[$suc_lang . '_EDITED'] . $errormsgs . adm_back_link($this->u_action), $level);
		}
		else
		{
			trigger_error($user->lang[$suc_lang . '_ADDED'] . $errormsgs .adm_back_link($this->u_action), $level);
		}
	}
	
	function delete_badge($id, $mode)
	{
		global $db, $user, $auth, $template, $cache;
		global $config, $phpbb_root_path, $phpbb_admin_path, $phpEx;
		
		$icon_id = request_var('id', 0);
		$notice = '';
		
		if (confirm_box(true))
		{
			$sql = 'DELETE FROM ' . BADGES_TABLE . "
				WHERE badge_id = $icon_id";
			$db->sql_query($sql);
			
			$sql = 'DELETE FROM ' . BADGES_USERS_TABLE . "
				WHERE badge_id = $icon_id";
			$db->sql_query($sql);

			$notice = $user->lang['BADGE_DELETED'];

			$cache->destroy('_badges');
			$cache->destroy('_badge_images');
			$cache->destroy('sql', BADGES_TABLE);
		}
		else
		{
			confirm_box(false, $user->lang['CONFIRM_OPERATION'], build_hidden_fields(array(
				'i'			=> $id,
				'mode'		=> $mode,
				'id'		=> $icon_id,
				'action'	=> 'delete',
			)));
		}
		
		return $notice;
	}
	
	function move_up_down($action)
	{
		global $db, $user, $auth, $template, $cache;
		global $config, $phpbb_root_path, $phpbb_admin_path, $phpEx;
		
		$icon_id = request_var('id', 0);
		
		// Get current order id...
		$sql = 'SELECT badge_order as current_order
			FROM ' . BADGES_TABLE . "
			WHERE badge_id = $icon_id";
		$result = $db->sql_query($sql);
		$current_order = (int) $db->sql_fetchfield('current_order');
		$db->sql_freeresult($result);

		if ($current_order == 0 && $action == 'move_up')
		{
			return;
		}

		// on move_down, switch position with next order_id...
		// on move_up, switch position with previous order_id...
		$switch_order_id = ($action == 'move_down') ? $current_order + 1 : $current_order - 1;

		//
		$sql = 'UPDATE ' . BADGES_TABLE . "
			SET badge_order = $current_order
			WHERE badge_order = $switch_order_id
				AND badge_id <> $icon_id";
		$db->sql_query($sql);

		// Only update the other entry too if the previous entry got updated
		if ($db->sql_affectedrows())
		{
			$sql = 'UPDATE ' . BADGES_TABLE . "
				SET badge_order = $switch_order_id
				WHERE badge_order = $current_order
					AND badge_id = $icon_id";
			$db->sql_query($sql);
		}

		$cache->destroy('_badges');
		$cache->destroy('sql', BADGES_TABLE);
	}
	
	function show_user_badges($user_id, $group_id, $notice = '')
	{
		global $db, $user, $auth, $template, $cache;
		global $config, $phpbb_root_path, $phpbb_admin_path, $phpEx;
		
		include_once($phpbb_root_path . 'includes/functions_badges.' . $phpEx);
		
		$imageset = badge_imageset();
		
		$sql = 'SELECT ug.group_id
			FROM ' . USER_GROUP_TABLE . " ug
			WHERE ug.user_id = $user_id
				AND ug.user_pending = 0";
		$result = $db->sql_query($sql);

		$group_ary = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$group_ary[$row['group_id']] = $row['group_id'];
		}
		$db->sql_freeresult($result);
		
		$sql = 'SELECT ub.badge_id, ub.multiple
			FROM ' . BADGES_USERS_TABLE . " ub
			WHERE ub.user_id = $user_id";
		$result = $db->sql_query($sql);
		
		$user_ary = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$user_ary[$row['badge_id']] = $row['multiple'];
		}
		$db->sql_freeresult($result);
		
		$sql = 'SELECT b.*
			FROM ' . BADGES_TABLE . ' b
			ORDER BY b.badge_order';
		$result = $db->sql_query($sql);
		
		$badge_ary = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$badge_ary[$row['badge_id']] = $row;
		}
		$result = $db->sql_freeresult($result);
		
		// Generate user badges lists
		$badges = array('group' => array(), 'user' => array());
		foreach ($badge_ary as $badge)
		{
			if (isset($group_ary[$badge['badge_group']]))
			{
				if ($badge['badge_group_default_only'] == 0 || $badge['badge_group'] == $group_id) {
					$badges['group'][$badge['badge_id']] = $badge;
				}
			}
			
			if (isset($user_ary[$badge['badge_id']]))
			{
				$badges['user'][$badge['badge_id']] = $badge;
				$badges['user'][$badge['badge_id']]['multiple'] = $user_ary[$badge['badge_id']];
				unset($badge_ary[$badge['badge_id']]);
			}
		}
		
		// Process output
		$current_type = '';
		foreach ($badges as $badge_type => $data_ary)
		{
			if ($current_type != $badge_type)
			{
				$template->assign_block_vars('badge', array(
					'S_NEW_BADGE_TYPE'		=> true,
					'BADGE_TYPE'			=> $user->lang['USER_BADGE_' . strtoupper($badge_type)],
				));
				$current_type = $badge_type;
			}
			
			foreach ($data_ary as $data)
			{
				$multiple = (isset($data['multiple'])) ? $data['multiple'] : 0;
				$imglist = join(' ', get_badge_imageset($imageset[$data['badge_url']], $data, $multiple));
				// <img src="{badgelist.IMG_SRC}" width="{badgelist.WIDTH}" height="{badgelist.HEIGHT}" alt="{badgelist.ALT_TEXT}" title="{badgelist.TITLE_TEXT}" />
				
				$template->assign_block_vars('badge', array(
					'ALT_TEXT'		=> $data['badge_alt'],
					'TITLE_TEXT'	=> $data['badge_title'],
					'IMG_SRC'		=> $phpbb_root_path . $config['badge_path'] . '/' . $data['badge_url'],
					'IMG_SET'		=> $imglist,
					'WIDTH'			=> $data['badge_width'],
					'HEIGHT'		=> $data['badge_height'],
					'MULTIPLE'		=> (isset($data['multiple'])) ? $data['multiple'] : '-',
					'U_EDIT'		=> $this->u_action . "&amp;action=edit&amp;u=$user_id&amp;id=" . $data['badge_id'],
					'U_DELETE'		=> $this->u_action . "&amp;action=delete&amp;u=$user_id&amp;id=" . $data['badge_id'],
					'S_BADGE_TYPE'	=> $badge_type,
					)
				);
			}
		}
		
		foreach ($badge_ary as $data)
		{
			$template->assign_block_vars('badgelist', array(
				'IMG_ID'		=> $data['badge_id'],
				'ALT_TEXT'		=> $data['badge_alt'],
				'TITLE_TEXT'	=> $data['badge_title'],
				'IMG_SRC'		=> $phpbb_root_path . $config['badge_path'] . '/' . $data['badge_url'],
				'WIDTH'			=> $data['badge_width'],
				'HEIGHT'		=> $data['badge_height'],
				)
			);
		}
		
		$template->assign_vars(array(
			'NOTICE'		=> $notice,
			'U_ACTION'		=> $this->u_action . "&amp;action=add&amp;u=$user_id",
		));
	}
	
	function show_edit_user_badge($user_id)
	{
		global $db, $user, $auth, $template, $cache;
		global $config, $phpbb_root_path, $phpbb_admin_path, $phpEx;
		
		$icon_id = request_var('id', 0);
		
		$sql = 'SELECT ub.badge_id, ub.multiple
			FROM ' . BADGES_USERS_TABLE . " ub
			WHERE ub.user_id = $user_id
				AND ub.badge_id = $icon_id";
		$result = $db->sql_query($sql);
		
		$binding = $db->sql_fetchrow($result);
		if ($binding === false)
		{
			trigger_error($user->lang['BADGE_NO_BINDING'] . adm_back_link($this->u_action), E_USER_WARNING);
		}
		$db->sql_freeresult($result);
		
		$sql = 'SELECT b.*
			FROM ' . BADGES_TABLE . ' b
			WHERE badge_id = ' . $icon_id;
		$result = $db->sql_query($sql);
		
		$badge = $db->sql_fetchrow($result);
		if ($badge === false)
		{
			trigger_error($user->lang['BADGE_NO_BADGE'] . adm_back_link($this->u_action), E_USER_WARNING);
		}
		$db->sql_freeresult($result);
		
		$template->assign_vars(array(
			'S_EDIT'		=> true,
			'U_ACTION'		=> $this->u_action . "&amp;action=modify&amp;u=$user_id&amp;id=$icon_id",
			'U_BACK'		=> $this->u_action . "&amp;u=$user_id",
			
			'ALT_TEXT'		=> $badge['badge_alt'],
			'TITLE_TEXT'	=> $badge['badge_title'],
			'IMG_SRC'		=> $phpbb_root_path . $config['badge_path'] . '/' . $badge['badge_url'],
			'WIDTH'			=> $badge['badge_width'],
			'HEIGHT'		=> $badge['badge_height'],
			'MULTIPLE'		=> $binding['multiple'],
		));
	}
	
	function modify_user_badge($user_id)
	{
		global $db, $user, $auth, $template, $cache;
		global $config, $phpbb_root_path, $phpbb_admin_path, $phpEx;
		
		$icon_id = request_var('id', 0);
		
		$sql = 'SELECT ub.badge_id, ub.multiple
			FROM ' . BADGES_USERS_TABLE . " ub
			WHERE ub.user_id = $user_id
				AND ub.badge_id = $icon_id";
		$result = $db->sql_query($sql);
		
		$binding = $db->sql_fetchrow($result);
		if ($binding === false)
		{
			trigger_error($user->lang['BADGE_NO_BINDING'] . adm_back_link($this->u_action), E_USER_WARNING);
		}
		$db->sql_freeresult($result);
		
		$sql = 'SELECT b.*
			FROM ' . BADGES_TABLE . ' b
			WHERE badge_id = ' . $icon_id;
		$result = $db->sql_query($sql);
		
		$badge = $db->sql_fetchrow($result);
		if ($badge === false)
		{
			trigger_error($user->lang['BADGE_NO_BADGE'] . adm_back_link($this->u_action), E_USER_WARNING);
		}
		$db->sql_freeresult($result);
		
		$multiple = (isset($_POST['multiple'])) ? request_var('multiple', 0) : 1;
		
		$error = validate_data(
			array('multiple'	=> $multiple),
			array('multiple'	=> array('num', false, 1),
		));
		
		$badges_edited = 0;
		if (empty($error))
		{
			$sql = 'UPDATE ' . BADGES_USERS_TABLE . "
				SET multiple = $multiple
				WHERE user_id = $user_id
					AND badge_id = $icon_id";
			$db->sql_query($sql);
			$badges_edited++;
		}
		
		switch ($badges_edited)
		{
			case 0:
				$suc_lang = 'BADGE_NONE_EDITED';
				$level = E_USER_WARNING;
			break;
			
			case 1:
				$suc_lang = 'BADGE_ONE_EDITED';
				$level = E_USER_NOTICE;
			break;
			
			default:
				$suc_lang = 'BADGE_EDITED';
				$level = E_USER_NOTICE;
		}
		
		$error = preg_replace('#^([A-Z_]+)$#e', "(!empty(\$user->lang['\\1'])) ? \$user->lang['\\1'] : '\\1'", $error);
		if (sizeof($error) > 0)
		{
			$error = '<br />' . join('<br />', $error);
		}
		else
		{
			$error = '';
		}
		
		trigger_error($user->lang[$suc_lang] . $error . adm_back_link($this->u_action . "&amp;u=$user_id"), $level);
	}
	
	function add_user_badges($user_id)
	{
		global $db, $user, $auth, $template, $cache;
		global $config, $phpbb_root_path, $phpbb_admin_path, $phpEx;
		
		$badge_add = (isset($_POST['add_img'])) ? request_var('add_img', array(0 => 0)) : array();
		
		$sql = 'SELECT ub.badge_id
			FROM ' . BADGES_USERS_TABLE . " ub
			WHERE ub.user_id = $user_id";
		$result = $db->sql_query($sql);
		
		$user_ary = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$user_ary[$row['badge_id']] = $row['badge_id'];
		}
		$db->sql_freeresult($result);
		
		$sql = 'SELECT b.*
			FROM ' . BADGES_TABLE . ' b
			ORDER BY b.badge_order';
		$result = $db->sql_query($sql);
		
		$badge_ary = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$badge_ary[$row['badge_id']] = $row;
		}
		$result = $db->sql_freeresult($result);
		
		$badges_added = 0;
		foreach ($badge_add as $b_id => $blank)
		{
			if (isset($badge_ary[$b_id]) && !isset($user_ary[$b_id]))
			{
				$insert_sql = array(
					'badge_id'	=> $b_id,
					'user_id'	=> $user_id,
				);
				
				$sql = 'INSERT INTO ' . BADGES_USERS_TABLE . $db->sql_build_array('INSERT', $insert_sql);
				$db->sql_query($sql);
				$badges_added++;
			}
		}
		
		$level = E_USER_NOTICE;
		switch ($badges_added)
		{
			case 0:
				$suc_lang = 'BADGE_NONE_ADDED';
				$level = E_USER_WARNING;
			break;
				
			case 1:
				$suc_lang = 'BADGE_ONE_ADDED';
			break;
				
			default:
				$suc_lang = 'BADGE_ADDED';
		}
		
		trigger_error($user->lang[$suc_lang] . adm_back_link($this->u_action . "&amp;u=$user_id"), $level);
	}
	
	function delete_user_badge($id, $mode, $user_id)
	{
		global $db, $user, $auth, $template, $cache;
		global $config, $phpbb_root_path, $phpbb_admin_path, $phpEx;
		
		$icon_id = request_var('id', 0);
		$notice = '';
		
		if (confirm_box(true))
		{
			$sql = 'DELETE FROM ' . BADGES_USERS_TABLE . "
				WHERE badge_id = $icon_id
					AND user_id = $user_id";
			$db->sql_query($sql);

			$notice = $user->lang['BADGE_DELETED'];
		}
		else
		{
			confirm_box(false, $user->lang['CONFIRM_OPERATION'], build_hidden_fields(array(
				'i'			=> $id,
				'mode'		=> $mode,
				'id'		=> $icon_id,
				'action'	=> 'delete',
			)));
		}
		
		return $notice;
	}
}

?>